import logging
from dataclasses import dataclass
from typing import Union

import httpx
from telethon import events

from bot_s3rius.config import config
from bot_s3rius.utils.library_package import get_library_info, render_package_info
from bot_s3rius.utils.responses import failed_search_answer, mark_unread

logger = logging.getLogger(__name__)


@dataclass
class PackageIndex:
    name: str
    install_cmd: Union[str, callable]


@config.telegram_client.on(events.NewMessage(pattern=r"^\.pip (.*)"))
@mark_unread
async def find_py_package(event: events.NewMessage.Event):
    package_name = event.pattern_match.group(1).strip()
    async with httpx.AsyncClient() as client:
        response = await client.get(f"https://pypi.org/pypi/{package_name}/json")
    if response.status_code != 200:
        await event.reply(failed_search_answer())
        return
    package_data = response.json()
    info = package_data["info"]
    response_str = render_package_info(
        {
            "Current version": info["version"],
            "pypi url": info["package_url"],
            "Homepage": info["home_page"],
            "Required python": info["requires_python"],
        },
        f"pip install {package_name}=={info['version']}",
    )
    await event.reply(response_str)


@config.telegram_client.on(events.NewMessage(pattern=r"^\.cargo (.*)"))
@mark_unread
async def find_rs_package(event: events.NewMessage.Event):
    package_name = event.pattern_match.group(1).strip()
    async with httpx.AsyncClient() as client:
        response = await client.get(f"https://crates.io/api/v1/crates/{package_name}")
    if response.status_code != 200:
        await event.reply(failed_search_answer())
        return
    package_data = response.json()
    crate = package_data["crate"]
    response_str = render_package_info(
        {
            "Current version": crate["newest_version"],
            "Repository": crate["repository"],
            "Crates URL": f"https://crates.io/crates/{package_name}",
            "Docs": crate["documentation"],
            "Description": crate["description"],
        },
        f"cargo add {package_name}@{crate['newest_version']}",
    )

    await event.reply(response_str)


@config.telegram_client.on(events.NewMessage(pattern=r"^\.aur (.*)"))
@mark_unread
async def find_aur_package(event: events.NewMessage.Event):
    package_name = event.pattern_match.group(1).strip()
    async with httpx.AsyncClient() as client:
        response = await client.get(
            f"https://aur.archlinux.org/rpc/?v=5&type=info&arg[]={package_name}"
        )
    if response.status_code != 200:
        await event.reply(failed_search_answer())
        return
    response = response.json()
    if response["resultcount"] < 1:
        await event.reply(failed_search_answer())
        return
    pkg_info = response["results"][0]
    response_str = render_package_info(
        {
            "Current version": pkg_info["Version"],
            "Maintainer": pkg_info["Maintainer"],
            "Repository": pkg_info["URL"],
            "AUR URL": f"https://aur.archlinux.org/packages/{package_name}",
            "Description": pkg_info["Description"],
        },
        f"pikaur -Syu {package_name}",
    )
    await event.reply(response_str)


def mapper_function(index: PackageIndex):
    @mark_unread
    async def function(event):
        package_name = event.pattern_match.group(1).strip()
        response_str = await get_library_info(
            index.name, package_name, index.install_cmd
        )
        await event.reply(response_str)

    return function


def register_functions():
    package_mappers = {
        "nuget": PackageIndex("NuGet", "dotnet add package {pkg} --version {ver}"),
        "hackage": PackageIndex("Hackage", "- {pkg} >= {ver}"),
        "npm": PackageIndex("NPM", "npm i {pkg}@^{ver}"),
        "brew": PackageIndex("Homebrew", "brew install {pkg}"),
        "pub": PackageIndex("Pub", "dependencies:\n\t{pkg}: ^{ver}"),
    }

    for alias, mapper in package_mappers.items():
        config.telegram_client.add_event_handler(
            mapper_function(mapper), events.NewMessage(pattern=rf"^\.{alias} (.*)")
        )


register_functions()
